@@ -33,6 +33,7 @@ class GFrame:
         self.bg_color = (0, 0, 0)
         self.is_visible = True
         self.is_pressed = False
+        self.is_enabled = True
         self.catch_mouse = True
         self.catch_keyboard = True
         self.pressed_child = None
@@ -40,6 +41,10 @@ class GFrame:
         self.child = []
         self.z_order = z_order
         self.last_hover = None
+        self.wm_info = pygame.display.get_wm_info()
+        self.hwnd = self.wm_info.get('window') or 0
+        self.hdc = self.wm_info.get('hdc') or 0
+        self.hinstance = self.wm_info.get('hinstance') or 0
 
     def on_before_update(self) -> None:
         pass
@@ -93,7 +98,8 @@ class GFrame:
             else:
                 return self.pressed_child.on_mouse_move(x - self.scroll_x, y - self.scroll_y)
         for child in self.child[::-1]:
-            if not child.is_visible or not is_object_colliding(child, x - self.scroll_x, y - self.scroll_y):
+            if not child.is_visible or not child.is_enabled\
+                    or not is_object_colliding(child, x - self.scroll_x, y - self.scroll_y):
                 continue
             if not child == self.last_hover:
                 if self.last_hover:
@@ -119,14 +125,16 @@ class GFrame:
             self.is_global_pressed = True
             self.on_global_mouse_down(x, y, button)
         for child in self.child[::-1]:
-            if not child.is_visible or not is_object_colliding(child, x - self.scroll_x, y - self.scroll_y):
+            if not child.is_visible or not child.is_enabled\
+                    or not is_object_colliding(child, x - self.scroll_x, y - self.scroll_y):
                 continue
             child.is_pressed = True
             self.pressed_child = child
+            child.last_btn = button
             if child.catch_mouse:
-                return child.on_mouse_down_(x - self.scroll_x, y - self.scroll_y)
+                return child.on_mouse_down_(x - self.scroll_x, y - self.scroll_y, button)
             else:
-                return child.on_mouse_down(x - self.scroll_x, y - self.scroll_y)
+                return child.on_mouse_down(x - self.scroll_x, y - self.scroll_y, button)
         self.on_mouse_down(x, y, button)
 
     def on_global_mouse_up_(self, x: int, y: int, button: int) -> None:
@@ -137,18 +145,30 @@ class GFrame:
             return
         self.pressed_child.is_pressed = False
         if self.pressed_child.catch_mouse:
-            self.pressed_child.on_mouse_up_(x - self.scroll_x, y - self.scroll_y)
+            self.pressed_child.on_mouse_up_(x - self.scroll_x, y - self.scroll_y, button)
         else:
-            self.pressed_child.on_mouse_up(x - self.scroll_x, y - self.scroll_y)
+            self.pressed_child.on_mouse_up(x - self.scroll_x, y - self.scroll_y, button)
         self.pressed_child = None
         self.on_mouse_up(x, y, button)
 
+    def on_global_mouse_leave(self, key_dict: dict) -> None:
+        pass
+
+    def on_global_mouse_enter(self, key_dict: dict) -> None:
+        pass
+
     def on_key_down(self, key_dict: dict) -> None:
         pass
 
     def on_key_up(self, key_dict: dict) -> None:
         pass
 
+    def on_focus_leave(self, key_dict: dict) -> None:
+        pass
+
+    def on_focus_enter(self, key_dict: dict) -> None:
+        pass
+
     def on_key_down_(self, key_dict: dict) -> None:
         for child in self.child:
             if child.catch_keyboard and child.on_key_down_(key_dict):
@@ -209,6 +229,18 @@ class GFrame:
             if event.type == pygame.QUIT:
                 self.on_quit()
                 continue
+            if event.type == pygame.WINDOWLEAVE:
+                self.on_global_mouse_leave(event.dict)
+                continue
+            if event.type == pygame.WINDOWENTER:
+                self.on_global_mouse_enter(event.dict)
+                continue
+            if event.type == pygame.WINDOWFOCUSLOST:
+                self.on_focus_leave(event.dict)
+                continue
+            if event.type == pygame.WINDOWFOCUSGAINED:
+                self.on_focus_enter(event.dict)
+                continue
 
     def resize(self, w: int, h: int) -> pygame.Surface:
         if self.is_main_window:
